package eu.dnetlib.data.mapping.cleaner;

import java.io.StringReader;

import org.apache.commons.lang3.BooleanUtils;
import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.dnetlib.errors.CleanerException;
import eu.dnetlib.errors.InformationServiceException;
import eu.dnetlib.is.resource.SimpleResourceService;
import eu.dnetlib.is.vocabulary.VocabularyService;

@Service
public class CleanerFactory {

	@Autowired
	private SimpleResourceService simpleResourceService;

	@Autowired
	private VocabularyService vocabularyService;

	public Cleaner obtainCleaningRule(final String ruleId) throws CleanerException {
		String prof;
		try {
			prof = simpleResourceService.getContent(ruleId);
		} catch (final InformationServiceException e) {
			try {
				prof = simpleResourceService.getContent("cleaning_rule", ruleId);
			} catch (final InformationServiceException e1) {
				throw new CleanerException("Cleaning rule not found: " + ruleId);
			}
		}

		try {
			final SAXReader reader = new SAXReader();
			final Document doc = reader.read(new StringReader(prof));

			final Cleaner cleaner = new Cleaner();

			for (final Node node : doc.selectNodes("//RULE")) {
				final String xpath = node.valueOf("@xpath");
				final String voc = node.valueOf("@vocabulary");
				final boolean strict = BooleanUtils.toBoolean(node.valueOf("@strict"));

				final VocabularyRule rule = new VocabularyRule(xpath, strict, voc, vocabularyService.listTerms(voc));

				cleaner.getRules().add(rule);
			}
			return cleaner;
		} catch (final Exception e) {
			throw new CleanerException("Error obtaing cleaner rule " + ruleId, e);
		}
	}

}
