package eu.dnetlib.data.mapping.xslt;

import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

import javax.xml.transform.stream.StreamSource;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.dnetlib.data.mapping.RecordTransformer;
import eu.dnetlib.errors.TransformationException;
import eu.dnetlib.is.resource.SimpleResourceService;
import net.sf.saxon.s9api.ExtensionFunction;
import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.QName;
import net.sf.saxon.s9api.SaxonApiException;
import net.sf.saxon.s9api.Serializer;
import net.sf.saxon.s9api.XdmAtomicValue;
import net.sf.saxon.s9api.XdmNode;
import net.sf.saxon.s9api.XsltCompiler;
import net.sf.saxon.s9api.XsltExecutable;
import net.sf.saxon.s9api.XsltTransformer;

@Service
public class XsltTransformFactory {

	public static final String QNAME_BASE_URI = "http://eu/dnetlib/transform";

	private static final Log log = LogFactory.getLog(XsltTransformFactory.class);

	@Autowired
	private SimpleResourceService simpleResourceService;

	@Autowired
	private List<DnetXsltFunction> xsltFunctions;

	public RecordTransformer<String, String> getTransformer(final String ruleId, final Map<String, String> initialParams) throws TransformationException {
		try {
			final String xsltText = simpleResourceService.getContent(ruleId);

			final Processor processor = new Processor(false);
			xsltFunctions.forEach(f -> {
				final ExtensionFunction extFunction = f.asExtensionFunction();
				processor.registerExtensionFunction(extFunction);
				log.info("New XSLT function registered: " + extFunction.getName());
			});

			final XsltCompiler comp = processor.newXsltCompiler();
			initialParams.forEach((k, v) -> {
				comp.setParameter(new QName(k), new XdmAtomicValue(v));
			});

			final XsltExecutable xslt = comp
				.compile(new StreamSource(IOUtils.toInputStream(xsltText, StandardCharsets.UTF_8)));

			return xml -> {
				try {
					final XdmNode source = processor
						.newDocumentBuilder()
						.build(new StreamSource(IOUtils.toInputStream(xml, StandardCharsets.UTF_8)));

					final XsltTransformer trans = xslt.load();
					trans.setInitialContextNode(source);

					final StringWriter output = new StringWriter();
					final Serializer out = processor.newSerializer(output);
					out.setOutputProperty(Serializer.Property.METHOD, "xml");
					out.setOutputProperty(Serializer.Property.INDENT, "yes");

					trans.setDestination(out);
					trans.transform();

					return output.toString();
				} catch (final SaxonApiException e) {
					throw new RuntimeException(e);
				}
			};

		} catch (final Throwable e) {
			throw new TransformationException("error preparing xslt transfomer", e);
		}
	}

}
