package eu.dnetlib.data.mdstore;

import java.util.Iterator;
import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.dnetlib.data.mdstore.model.MDStoreWithInfo;
import eu.dnetlib.data.mdstore.model.records.MetadataRecord;
import eu.dnetlib.errors.MDStoreManagerException;

@Service
public class MDStoreStreamReader {

	@Autowired
	private MDStoreService mdStoreService;

	private enum Status {
		PREPARED,
		READING,
		COMPLETED,
		FAILED
	}

	// TODO the failure could be throw consuming the stream, so it is necessary to perform a refactoring of this method
	public Stream<MetadataRecord> prepareMDStoreStream(final String mdstoreId) throws MDStoreManagerException {

		final MDStoreWithInfo mdstore = mdStoreService.findMdStore(mdstoreId);
		final Iterator<? extends MetadataRecord> innerIterator = mdStoreService.streamVersionRecords(mdstore.getCurrentVersion()).iterator();

		return StreamSupport.stream(Spliterators.spliteratorUnknownSize(new Iterator<MetadataRecord>() {

			private Status status = Status.PREPARED;

			@Override
			public boolean hasNext() {
				if (innerIterator.hasNext()) {
					return true;
				} else {
					try {
						complete();
						return false;
					} catch (final MDStoreManagerException e) {
						throw new RuntimeException("Error reading mdstore", e);
					}
				}
			}

			@Override
			public MetadataRecord next() {
				try {
					verifyStart();
					return innerIterator.next();
				} catch (final Throwable e) {
					try {
						fail();
						throw new RuntimeException("Error reading mdstore", e);
					} catch (final MDStoreManagerException e1) {
						throw new RuntimeException("Error reading mdstore", e);
					}
				}
			}

			private synchronized void verifyStart() throws MDStoreManagerException {
				if (status == Status.PREPARED) {
					status = Status.READING;
					mdStoreService.startReading(mdstoreId);
				}
			}

			private synchronized void complete() throws MDStoreManagerException {
				if (status == Status.PREPARED || status == Status.READING) {
					status = Status.COMPLETED;
					mdStoreService.endReading(mdstoreId);
				}
			}

			private synchronized void fail() throws MDStoreManagerException {
				if (status == Status.PREPARED || status == Status.READING) {
					status = Status.FAILED;
					mdStoreService.endReading(mdstoreId);
				}
			}
		}, 0), false);
	}

}
