package eu.dnetlib.data.mdstore.model.records;

import java.io.Serializable;

import org.apache.commons.codec.digest.DigestUtils;

/**
 * This class models a record in a Metadata store collection
 */
public interface MetadataRecord extends Serializable {

	/** The D-Net Identifier associated to the record */
	public String getId();

	public void setId(final String id);

	/** The original Identifier of the record */
	public String getOriginalId();

	public void setOriginalId(final String originalId);

	/** The encoding of the record, should be JSON or XML */
	public String getEncoding();

	public void setEncoding(final String encoding);

	/**
	 * The information about the provenance of the record see @{@link Provenance} for the model of this information
	 */
	public Provenance getProvenance();

	public void setProvenance(final Provenance provenance);

	/** The content of the metadata */
	public String getBody();

	public void setBody(final String body);

	/** the date when the record has been stored */
	public Long getDateOfCollection();

	public void setDateOfCollection(final Long dateOfCollection);

	/** the date when the record has been stored */
	public Long getDateOfTransformation();

	public void setDateOfTransformation(final Long dateOfTransformation);

	public static String generateIdentifier(final String originalId, final String nsPrefix) {
		return String.format("%s::%s", nsPrefix, DigestUtils.md5Hex(originalId));
	}

}
