package eu.dnetlib.manager.history;

import java.time.Instant;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.TimeZone;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import eu.dnetlib.manager.history.model.WfHistoryEntry;
import eu.dnetlib.manager.history.repository.WfHistoryEntryRepository;

@Service
public class WorkflowLogger {

	@Autowired
	private WfHistoryEntryRepository wfHistoryEntryRepository;

	public List<WfHistoryEntry> history(final int total, final Long from, final Long to) {
		if (from == null && to == null) {
			return wfHistoryEntryRepository.findAll(PageRequest.of(0, total, Sort.by("endDate").descending())).toList();
		} else {

			final LocalDateTime fromTime = from != null ? LocalDateTime.ofInstant(Instant.ofEpochMilli(from), TimeZone
				.getDefault()
				.toZoneId()) : LocalDateTime.MIN;
			final LocalDateTime toTime = to != null ? LocalDateTime.ofInstant(Instant.ofEpochMilli(to), TimeZone
				.getDefault()
				.toZoneId()) : LocalDateTime.MAX;

			return wfHistoryEntryRepository.findByEndDateBetweenOrderByEndDateDesc(fromTime, toTime);
		}
	}

	public List<WfHistoryEntry> history(final String wfConfId) {
		return wfHistoryEntryRepository.findByWfConfigurationIdOrderByEndDateDesc(wfConfId);
	}

	public WfHistoryEntry getLog(final String processId) {
		return wfHistoryEntryRepository.findById(processId).get();
	}

	public void saveLog(final WfHistoryEntry pe) {
		wfHistoryEntryRepository.save(pe);
	}

	public Optional<WfHistoryEntry> getLastLogForConfiguration(final String id) {
		return wfHistoryEntryRepository.findFirstByWfConfigurationIdOrderByEndDateDesc(id);
	}

}
