package eu.dnetlib.manager.wf.nodes.store;

import org.springframework.beans.factory.annotation.Autowired;

import eu.dnetlib.data.mdstore.MDStoreService;
import eu.dnetlib.data.mdstore.model.MDStoreType;
import eu.dnetlib.data.mdstore.model.MDStoreWithInfo;
import eu.dnetlib.data.mdstore.model.records.MetadataRecord;
import eu.dnetlib.errors.MDStoreManagerException;
import eu.dnetlib.manager.wf.annotations.WfInputParam;
import eu.dnetlib.manager.wf.annotations.WfNode;
import eu.dnetlib.manager.wf.annotations.WfOutputParam;
import eu.dnetlib.manager.wf.nodes.stream.StreamConsumerNode;

@WfNode("feedMdStore")
public class MdStoreFeederNode extends StreamConsumerNode<MetadataRecord> {

	@WfInputParam
	private String mdId;

	@WfInputParam
	private String storingType;

	@WfOutputParam
	private long total;

	@Autowired
	private MDStoreService mdStoreService;

	private String newVersion;
	private MDStoreType mdStoreType;

	@Override
	protected void init() throws Exception {
		final MDStoreWithInfo md = mdStoreService.findMdStore(mdId);
		mdStoreType = md.getType();

		if (storingType.equals(MDStoreService.REFRESH_MODE)) {
			newVersion = mdStoreService.prepareMdStoreVersion(mdId).getId();
		} else if (storingType.equals(MDStoreService.INCREMENTAL_MODE)) {
			newVersion = md.getCurrentVersion();
		} else {
			throw new MDStoreManagerException("Invalid mode " + storingType);
		}
	}

	@Override
	protected void consumeRecord(final MetadataRecord input) throws Exception {
		mdStoreService.addRecord(newVersion, mdStoreType, input);
	}

	@Override
	protected void complete() throws Exception {
		total = mdStoreService.commitMdStoreVersion(newVersion, mdStoreType);
	}
}
