package eu.dnetlib.manager.wf.nodes.store;

import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import eu.dnetlib.data.mdstore.MDStoreService;
import eu.dnetlib.data.mdstore.model.MDStoreVersion;
import eu.dnetlib.data.mdstore.model.records.MetadataRecord;
import eu.dnetlib.manager.wf.annotations.WfInputParam;
import eu.dnetlib.manager.wf.annotations.WfNode;
import eu.dnetlib.manager.wf.nodes.stream.StreamSupplierNode;

@WfNode("fetchMdStore")
public class MdStoreFetcherNode extends StreamSupplierNode<MetadataRecord> {

	@WfInputParam
	private String mdId;

	@Autowired
	private MDStoreService mdStoreService;

	private String currentVersion;

	@Override
	protected Stream<MetadataRecord> prepareStream() throws Exception {
		if (StringUtils.isNotBlank(currentVersion)) {
			return mdStoreService.streamVersionRecords(currentVersion);
		} else {
			return Stream.empty();
		}
	}

	@Override
	protected void before() throws Exception {
		final MDStoreVersion version = mdStoreService.startReading(mdId);
		this.currentVersion = version.getId();
	}

	@Override
	protected void after() throws Exception {
		if (StringUtils.isNotBlank(currentVersion)) {
			mdStoreService.endReading(currentVersion);
		}
	}

	@Override
	protected void abort() throws Exception {
		if (StringUtils.isNotBlank(currentVersion)) {
			mdStoreService.endReading(currentVersion);
		}
	}
}
