package eu.dnetlib.manager.wf.nodes.stream;

import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.dnetlib.manager.wf.workflows.util.SimpleCallable;

public class DnetStream<T> {

	private static final Log log = LogFactory.getLog(DnetStream.class);

	private final Stream<T> stream;
	private final SimpleCallable before;
	private final SimpleCallable after;
	private final SimpleCallable abort;

	public DnetStream(final Stream<T> stream, final SimpleCallable before, final SimpleCallable after, final SimpleCallable abort) {
		this.stream = stream;
		this.before = before;
		this.after = after;
		this.abort = abort;
	}

	public DnetStream<T> filter(final Predicate<T> predicate) {
		return new DnetStream<>(stream.filter(predicate), before, after, abort);
	}

	public <K> DnetStream<K> map(final Function<T, K> mapper) {
		return new DnetStream<>(stream.map(mapper), before, after, abort);
	}

	public void consume(final Consumer<T> action) {
		try {
			if (before != null) {
				before.call();
			}

			stream.forEach(action);

			if (after != null) {
				after.call();
			}
		} catch (final Throwable e) {
			log.error("Error reading stream: " + e);
			try {
				if (abort != null) {
					abort.call();
				}
			} catch (final Exception e1) {
				log.error("Error aborting stream reading: " + e1);
			}
			throw new RuntimeException(e);
		}
	}

}
