package eu.dnetlib.manager.wf.nodes.transfrom;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.springframework.beans.factory.annotation.Autowired;

import eu.dnetlib.data.mdstore.model.records.MetadataRecord;
import eu.dnetlib.data.mdstore.model.records.MetadataRecordImpl;
import eu.dnetlib.data.mdstore.model.records.Provenance;
import eu.dnetlib.dsm.DsmService;
import eu.dnetlib.dsm.model.Api;
import eu.dnetlib.dsm.model.Datasource;
import eu.dnetlib.manager.wf.annotations.WfInputParam;
import eu.dnetlib.manager.wf.annotations.WfNode;
import eu.dnetlib.manager.wf.nodes.stream.StreamMapperNode;

@WfNode("openaireMdBuilder")
public class OpenaireMdBuilderNode extends StreamMapperNode<String, MetadataRecord> {

	@WfInputParam
	private Datasource ds;
	@WfInputParam
	private Api api;
	@WfInputParam
	private boolean inferred;
	@WfInputParam
	private boolean deletedbyinference;
	@WfInputParam
	private String inferenceprovenance;
	@WfInputParam
	private float trust;
	@WfInputParam
	private String provenanceactionclassname;
	@WfInputParam
	private String provenanceactionclassid;

	@Autowired
	private DsmService dsmService;

	private String xpath;

	private long now;

	private Provenance provenance;

	@Override
	protected void init() throws Exception {
		xpath = api.getMetadataIdentifierPath();

		provenance = new Provenance();
		provenance.setDatasourceId(ds.getId());
		provenance.setDatasourceName(ds.getOfficialname());
		provenance.setNsPrefix(ds.getNamespaceprefix());

		now = System.currentTimeMillis();
	}

	@Override
	protected MetadataRecord mapRecord(final String xml) throws Exception {
		try {
			final Document doc = DocumentHelper.parseText(xml);
			final String origId = doc.valueOf(xpath);

			// TODO populate the xml record with missing oaf fields, see:
			// https://svn.driver.research-infrastructures.eu/driver/dnet45/modules/dnet-openaireplus-workflows/trunk/src/main/resources/eu/dnetlib/msro/openaireplus/workflows/repo-hi/xslt/openaireMdBuilder.xslt.st

			final MetadataRecord md = new MetadataRecordImpl();

			md.setId(MetadataRecord.generateIdentifier(origId, ds.getNamespaceprefix()));
			md.setOriginalId(origId);
			md.setBody(doc.asXML());
			md.setEncoding("XML");
			md.setDateOfCollection(now);
			md.setDateOfTransformation(null);
			md.setProvenance(provenance);

			return md;
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

}
