package eu.dnetlib.manager.wf.notification;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.dnetlib.manager.wf.model.NotificationCondition;
import eu.dnetlib.manager.wf.repository.WorkflowSubscriptionRepository;
import eu.dnetlib.manager.wf.workflows.procs.WorkflowProcess;
import eu.dnetlib.manager.wf.workflows.procs.WorkflowProcess.Status;
import eu.dnetlib.notifications.mail.EmailService;

@Service
public class EmailSender {

	private static final Log log = LogFactory.getLog(EmailSender.class);

	@Autowired
	private WorkflowSubscriptionRepository wfSubscriptionRepository;

	@Autowired
	private EmailService emailService;

	public void sendMails(final WorkflowProcess proc) {

		wfSubscriptionRepository.findByWfConfigurationId(proc.getWfConfId()).forEach(s -> {
			if (s.getCondition() == NotificationCondition.ALWAYS ||
				s.getCondition() == NotificationCondition.ONLY_FAILED && proc.getStatus() == Status.failure ||
				s.getCondition() == NotificationCondition.ONLY_SUCCESS && proc.getStatus() == Status.success) {
				try {
					final Map<String, Object> params = new HashMap<>();

					emailService.sendStoredMail(s.getEmail(), s.getMessageId(), params);

				} catch (final Exception e) {
					log.error("Error sending mail to " + s.getEmail(), e);
				}
			}

		});
	}
}
