package eu.dnetlib.manager.wf.workflows.procs;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import eu.dnetlib.errors.WorkflowManagerException;
import eu.dnetlib.is.model.resource.SimpleResource;
import eu.dnetlib.manager.wf.model.WorkflowConfiguration;
import eu.dnetlib.manager.wf.model.WorkflowTemplate;
import eu.dnetlib.manager.wf.workflows.graph.Graph;
import eu.dnetlib.manager.wf.workflows.graph.GraphLoader;
import eu.dnetlib.manager.wf.workflows.util.ProcessCallback;

@Component
public class ProcessFactory {

	private static final Log log = LogFactory.getLog(ProcessFactory.class);

	private String oldGeneratedId = "";

	private final DateTimeFormatter processIdFormatter = DateTimeFormatter.ofPattern("yyyyMMdd_HHmmss_S");

	@Autowired
	private GraphLoader graphLoader;

	public WorkflowProcess newProcess(final SimpleResource wfMetadata,
		final WorkflowTemplate wfTemplate,
		final WorkflowConfiguration conf,
		final ProcessCallback callback) throws WorkflowManagerException {

		final Map<String, String> globalParams = new HashMap<>();
		globalParams.putAll(conf.getSystemParams());
		globalParams.putAll(conf.getUserParams());

		final Graph graph = graphLoader.loadGraph(wfTemplate, globalParams);

		return new WorkflowProcess(generateProcessId(), wfMetadata, conf, graph, globalParams, callback);

	}

	private synchronized String generateProcessId() {
		String id = "";
		do {
			id = "wf_" + LocalDateTime.now().format(processIdFormatter);
			log.info("Generated processID " + id);
		} while (id.equals(oldGeneratedId));

		oldGeneratedId = id;

		return id;
	}
}
