package eu.dnetlib.manager.wf.workflows.procs;

import java.time.LocalDateTime;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;

import com.google.common.base.Throwables;

/**
 * Created by michele on 19/11/15.
 */
public class Token {

	private final String id;
	private final Env env = new Env();

	private String progressMessage;
	private final LocalDateTime startDate;
	private LocalDateTime endDate;

	private boolean failed;
	private boolean active;

	private String error = "";
	private String errorStackTrace = "";

	public Token() {
		this.id = "token-" + UUID.randomUUID();
		this.startDate = LocalDateTime.now();
		this.failed = false;
		this.active = true;
	}

	public String getId() {
		return this.id;
	}

	public Env getEnv() {
		return this.env;
	}

	public LocalDateTime getStartDate() {
		return this.startDate;
	}

	public LocalDateTime getEndDate() {
		return this.endDate;
	}

	public void setEndDate(final LocalDateTime endDate) {
		this.endDate = endDate;
	}

	public boolean isActive() {
		return this.active;
	}

	public void setActive(final boolean active) {
		this.active = active;
	}

	public boolean isFailed() {
		return this.failed;
	}

	public void setFailed(final boolean failed) {
		this.failed = failed;
	}

	public void release() {
		setEndDate(LocalDateTime.now());
		setActive(false);
	}

	public void releaseAsFailed(final Throwable e) {
		setEndDate(LocalDateTime.now());
		setActive(false);
		setFailed(true);
		setError(e.getMessage());
		setErrorStackTrace(Throwables.getStackTraceAsString(e));
	}

	public void releaseAsFailed(final String error) {
		setEndDate(LocalDateTime.now());
		setActive(false);
		setFailed(true);
		setError(error);
	}

	public void checkStatus() {
		if (isActive()) {
			if (StringUtils.isNotBlank(error)) {
				releaseAsFailed(error);
			} else {
				release();
			}
		}
	}

	public String getProgressMessage() {
		return progressMessage;
	}

	public void setProgressMessage(final String progressMessage) {
		this.progressMessage = progressMessage;
	}

	public String getError() {
		return this.error;
	}

	public void setError(final String error) {
		this.error = error;
	}

	public String getErrorStackTrace() {
		return this.errorStackTrace;
	}

	public void setErrorStackTrace(final String errorStackTrace) {
		this.errorStackTrace = errorStackTrace;
	}

}
